package com.brainysoftware.batik;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.batik.dom.svg.SVGDOMImplementation;
import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.apache.batik.util.XMLConstants;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class GraphServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	private final String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
	
	// Reference points for drawing
	private final int canvasWidth = 600;
	private final int canvasHeight = 400;
	private final int X_start = 30;
	private final int Y_start = 375;
	private final int X_end = 595;
	private final int Y_end = 10;
	
	// ASCII value of '0' character
	private final int asciiZero = 48;

	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		
		// Create the SVG Document
		DOMImplementation dom = SVGDOMImplementation.getDOMImplementation();
		Document doc = dom.createDocument(svgNS, "svg", null);
		
		// Get the root element, set canvas sizes
		Element root = doc.getDocumentElement();
		root.setAttributeNS(null, "width", "" + canvasWidth);
		root.setAttributeNS(null, "height", "" + canvasHeight);
		
		// Add font references
		ServletContext ctx  = getServletContext();
		File fontFile = new File(ctx.getRealPath("/ForgottenFuturist.svg"));
		File fontFileBold = new File(ctx.getRealPath("/ForgottenFuturistBold.svg"));
		Element defs = doc.createElementNS(svgNS, "defs");
		Element style = doc.createElementNS(svgNS, "style");
		style.setAttributeNS(null, "type", "text/css");
		CDATASection fontRef = doc.createCDATASection(
				"@font-face { font-family: 'ForgottenFuturist'; src: url(\"" + 
				fontFile.toURL().toString() + 
				"#ForgottenFuturist\") format(svg) }");
		CDATASection fontRefBold = doc.createCDATASection(
				"@font-face { font-family: 'ForgottenFuturistBold'; src: url(\"" + 
				fontFileBold.toURL().toString() + 
				"#ForgottenFuturistBold\") format(svg) }");
		style.appendChild(fontRef);
		style.appendChild(fontRefBold);
		defs.appendChild(style);
		
		// A vertical path to write a label for the vertical axis
		Element path = doc.createElementNS(svgNS, "path");
		path.setAttributeNS(null, "id", "path1");
		path.setAttributeNS(null, "d", "M 20 60 L 20 10");
		defs.appendChild(path);
		
		// Add a gradient fill
		Gradients.insertVerticalGradient(doc, "darkred");
		
		root.appendChild(defs);
		
		// Draw X and Y axes
		Element lineX = doc.createElementNS(svgNS, "line");
		
		lineX.setAttributeNS(null, "x1", "" + X_start);
		lineX.setAttributeNS(null, "y1", "" + Y_start);
		lineX.setAttributeNS(null, "x2", "" + X_end);
		lineX.setAttributeNS(null, "y2", "" + Y_start);
		lineX.setAttributeNS(null, "stroke", "black");
		lineX.setAttributeNS(null, "stroke-width", "2");
		
		Element lineY = doc.createElementNS(svgNS, "line");
		
		lineY.setAttributeNS(null, "x1", "" + X_start);
		lineY.setAttributeNS(null, "y1", "" + Y_start);
		lineY.setAttributeNS(null, "x2", "" + X_start);
		lineY.setAttributeNS(null, "y2", "" + Y_end);
		lineY.setAttributeNS(null, "stroke", "black");
		lineY.setAttributeNS(null, "stroke-width", "2");
		
		root.appendChild(lineX);
		root.appendChild(lineY);
		
		// Extract request parameters
		String sDay = req.getParameter("day");
		String sMonth = req.getParameter("month");
		String sYear = req.getParameter("year");
		String name = req.getParameter("name");
		
		// Don't allow the name to be empty
		if ("".equals(name)) name = "Anonymous";
		
		// Prepare the data for the graph
		char[] theData = (sMonth + sDay + sYear).toCharArray();
		
		// Draw the marks along the axes
		int incX = (X_end - X_start) / theData.length;
		
		for (int i = 1; i <= theData.length; i++) {
			Element stroke = doc.createElementNS(svgNS, "line");
			int movingX = X_start + incX * i;
			stroke.setAttributeNS(null, "x1", "" + movingX);
			stroke.setAttributeNS(null, "y1", "375");
			stroke.setAttributeNS(null, "x2", "" + movingX);
			stroke.setAttributeNS(null, "y2", "380");
			stroke.setAttributeNS(null, "stroke", "black");
			stroke.setAttributeNS(null, "stroke-width", "2");
			int textX = movingX - 3;
			Element markX = doc.createElementNS(svgNS, "text");
			markX.setAttributeNS(null, "x", "" + textX);
			markX.setAttributeNS(null, "y", "398");
			markX.setAttributeNS(null, "font-size", "22");
			markX.setAttributeNS(null, "font-family", "ForgottenFuturist");
			Text markXText = doc.createTextNode("" + i);
			markX.appendChild(markXText);
			
			root.appendChild(stroke);
			root.appendChild(markX);
		}
		
		int incY = (Y_start - Y_end) / 9;
		
		for (int i = 1; i <= 9; i++) {
			Element stroke = doc.createElementNS(svgNS, "line");
			int movingY = Y_start - incY * i;
			stroke.setAttributeNS(null, "x1", "30");
			stroke.setAttributeNS(null, "y1", "" + movingY);
			stroke.setAttributeNS(null, "x2", "25");
			stroke.setAttributeNS(null, "y2", "" + movingY);
			stroke.setAttributeNS(null, "stroke", "black");
			stroke.setAttributeNS(null, "stroke-width", "2");
			root.appendChild(stroke);
		}		
		
		// Create a label for Y axis (as text on a path)
		Element textPath = doc.createElementNS(svgNS, "textPath");
		textPath.setAttributeNS(XMLConstants.XLINK_NAMESPACE_URI, "href", "#path1");
		Text labelY = doc.createTextNode("Values");
		textPath.appendChild(labelY);
		
		Element textLabel = doc.createElementNS(svgNS, "text");
		textLabel.setAttributeNS(null, "font-size", "22");
		textLabel.setAttributeNS(null, "font-family", "ForgottenFuturist");
		textLabel.appendChild(textPath);
				
		root.appendChild(textLabel);
		
		// Create the greeting string
		Element greeting = doc.createElementNS(svgNS, "text");
		Text greetingText = doc.createTextNode("Hello " + name + "!");
		greeting.appendChild(greetingText);
		greeting.setAttributeNS(null, "x", "55");
		greeting.setAttributeNS(null, "y", "45");
		greeting.setAttributeNS(null, "font-size", "44");
		greeting.setAttributeNS(null, "font-family", "ForgottenFuturistBold");
		
		root.appendChild(greeting);
		
		// Create the graph
		StringBuffer graphPoints = new StringBuffer();
		graphPoints.append(X_start + "," + Y_start + " "); // The initial point 
		
		int posX = X_start;
		
		for (int i = 0; i < theData.length; i++) {
			posX = X_start + (i + 1) * incX;
			int posY = Y_start - (theData[i] - asciiZero) * incY;
			String point = posX + "," + posY + " ";
			graphPoints.append(point);
		}
		
		graphPoints.append(posX + "," + Y_start);
		
		Element graph = doc.createElementNS(svgNS, "polyline");
		graph.setAttributeNS(null, "fill", "url(#" + Gradients.VERTICAL_GRADIENT_ID + ")");
		graph.setAttributeNS(null, "stroke", "darkred");
		graph.setAttributeNS(null, "stroke-width", "2");
		graph.setAttributeNS(null, "points", graphPoints.toString());
		
		root.appendChild(graph);		
		
		// Set the content type of the response
		resp.setContentType("image/png");
				
		// Transcode the SVG Document into a PNG image
		// and send it as a response
		PNGTranscoder t = new PNGTranscoder();
		TranscoderInput input =	new TranscoderInput(doc);		
		TranscoderOutput output = new TranscoderOutput(resp.getOutputStream());
		try {
			t.transcode(input, output);
		}
		catch (TranscoderException te) {
			te.printStackTrace();
		}
    		resp.getOutputStream().close();
	}
}
